#ifndef STRING_TEST_H
#define STRING_TEST_H

#undef _DEBUG
#include <cassert>
#include <cstdlib>
#include <cstring>
#include <string>
#include <iterator>


#undef	TEST_OUT_OF_RANGE
#define TEST_OUT_OF_RANGE(a)		try { a; assert(0); } catch (std::out_of_range) {}
//x #define TEST_OUT_OF_RANGE(a)

#define T(a)			a
//#define T(a)			L##a

//#define UNUSE_THROW

#undef PRINTF
#define PRINTF(x)		printf x


template<class STRING>
class String_Test {
public:
	void test()
	{
		PRINTF(("-"));
		test_constructor();
		PRINTF(("\\"));
		test_compare();
		PRINTF(("|"));
		test_etc();
		PRINTF(("/"));
		test_assign();
		PRINTF(("-"));
		test_append();
		PRINTF(("\\"));
		test_join();
		PRINTF(("|"));
		test_erase();
		PRINTF(("/"));
		test_insert();
		PRINTF(("-"));
		test_replace();
		PRINTF(("\\"));
		test_find();
		PRINTF(("|"));
		test_rfind();
		PRINTF(("/"));
		test_find_first_of();
		PRINTF(("-"));
		test_find_last_of();
		PRINTF(("\\"));
		test_find_first_not_of();
		PRINTF(("|"));
		test_find_last_not_of();
		PRINTF(("/"));
		test_copy();
		PRINTF(("-"));
		test_swap();
		PRINTF(("\\"));
		test_substr();
		PRINTF(("OK\n"));
	}


private:
	typename STRING::iterator ind2ite(STRING& str, std::size_t no) const {
		typename STRING::iterator it(str.begin());
		std::advance(it, no);
		return it;
	}

	/// RXgN^, empty,size,capacity,max_size,length gp
	void test_constructor()
	{
		// ftHgERXgN^
		STRING	str0;
		assert (str0.empty());
		assert (str0.size() == 0);
		assert (str0.capacity() >= 0);
		assert (str0.max_size() >= str0.size());

		const STRING str1(T("0123456789"));
		//STRING	str1(10);
		assert (str1.empty() == false);
		assert (str1.size()   == 10);
		assert (str1.length() == 10);
		assert (str1.capacity() >= 10);
		assert (str1.max_size() >= str1.size());
		assert ( std::memcmp(str1.data() , T("0123456789"), 10) == 0 );
		assert ( std::memcmp(str1.c_str(), T("0123456789"), 11) == 0 );

		STRING	str2(10, T('@'));
		assert (str2.empty() == false);
		assert (str2.size() == 10);
		assert (str1.length() == 10);
		assert (str2.capacity() >= 10);
		assert ( std::memcmp(&str2[0], T("@@@@@@@@@@"), 10) == 0 );
		assert ( std::memcmp(str2.c_str(), T("@@@@@@@@@@"), 11) == 0 );
		assert (str2.max_size() >= str2.size());

		STRING	str3(str1);
		assert (str3.empty() == false);
		assert (str3.size() == 10);
		assert (str3.capacity() >= 10);
		assert ( std::memcmp(str3.data() , T("0123456789"), 10) == 0 );
		assert ( std::memcmp(str3.c_str(), T("0123456789"), 11) == 0 );
		assert (str3.max_size() >= str3.size());

		TEST_OUT_OF_RANGE( STRING	str4a(str1,100) );

		STRING	str4(str1, 2);
		assert (str4.empty() == false);
		assert (str4.size() == 8);
		assert (str4.capacity() >= 8);
		assert ( std::memcmp(str4.data() , T("23456789"), 8) == 0 );
		assert ( std::memcmp(str4.c_str(), T("23456789"), 9) == 0 );
		assert (str4.max_size() >= str4.size());

		TEST_OUT_OF_RANGE( STRING	str5a(str1, 100, 5) );

		STRING	str5(str1, 3, 5);
		assert (str5.empty() == false);
		assert (str5.size() == 5);
		assert (str5.capacity() >= 5);
		assert ( std::memcmp(str5.data() , T("34567"), 5) == 0 );
		assert ( std::memcmp(str5.c_str(), T("34567"), 6) == 0 );
		assert (str5.max_size() >= str5.size());

		STRING	str6(T("0123456789"), 6);
		assert (str6.empty() == false);
		assert (str6.size() == 6);
		assert (str6.capacity() >= 6);
		assert ( std::memcmp(str6.data(),  T("012345"), 6) == 0 );
		assert ( std::memcmp(str6.c_str(), T("012345"), 7) == 0 );
		assert (str6.max_size() >= str6.size());

		STRING	str7(str1.begin(), str1.end());
		assert (str7.empty() == false);
		assert (str7.size() == 10);
		assert (str7.capacity() >= 10);
		assert ( std::memcmp(str7.data(),  T("0123456789"), 10) == 0 );
		assert ( std::memcmp(str7.c_str(), T("0123456789"), 11) == 0 );
		assert (str7.max_size() >= str7.size());

		STRING	str8(T("abcd\0efghi"), 10);
		assert (str8.empty() == false);
		std::size_t l = str8.size();
		assert (l == 10);
		assert (str8.capacity() >= 10);
		assert ( std::memcmp(str8.data(),  T("abcd\0efghi"), 10) == 0 );
		assert ( std::memcmp(str8.c_str(), T("abcd"), 5) == 0 );
		assert (str8.max_size() >= str8.size());

		STRING	str9(str1.end(), str1.end());
		assert (str9.empty() != false);
		assert (str9.size() == 0);
		assert (str9.max_size() >= str9.size());
	}

	/// r
	void test_compare()
	{
		const STRING	str0;
		const STRING	str1(T("0123456789"));
		const STRING	str5(str1, 3, 5);
		const STRING	str6(T("abcdefghij"));
		const STRING	str7(T("abcdefghij"), 7);
		const STRING	str8(str1);

		assert (str1 != str0);
		assert ((str1 == str0) == 0);
		assert (str0 <= str1);
		assert (str0 <	str1);
		assert (str1 >= str0);
		assert (str1 >	str0);

		assert (str8 != str7);
		assert ((str8 == str7) == 0);
		assert (str8 <= str7);
		assert (str8 <	str7);
		assert (str7 >= str8);
		assert (str7 >	str8);

		assert (str6 != str7);
		assert ((str6 == str7) == 0);
		assert (str6 >= str7);
		assert (str6 >	str7);
		assert (str7 <= str6);
		assert (str7 <	str6);

		assert (str8 == str1);
		assert ((str8 != str1) == 0);
		assert ((str8 <= str1) != 0);
		assert ((str8 <  str1) == 0);
		assert ((str8 >= str1) != 0);
		assert ((str8 >  str1) == 0);

		assert ((str0 == T("")) != 0);
		assert ((str0 != T("")) == 0);
		assert ((str0 <= T("")) != 0);
		assert ((str0 <  T("")) == 0);
		assert ((str0 >= T("")) != 0);
		assert ((str0 >  T("")) == 0);

		assert ((str1 == T("")) == 0);
		assert ((str1 != T("")) != 0);
		assert ((str1 <= T("")) == 0);
		assert ((str1 <  T("")) == 0);
		assert ((str1 >= T("")) != 0);
		assert ((str1 >  T("")) != 0);

		assert ((str8 == T("0123456789")) != 0);
		assert ((str8 != T("0123456789")) == 0);
		assert ((str8 <= T("0123456789")) != 0);
		assert ((str8 <  T("0123456789")) == 0);
		assert ((str8 >= T("0123456789")) != 0);
		assert ((str8 >  T("0123456789")) == 0);

		assert ((T("") == str0) != 0);
		assert ((T("") != str0) == 0);
		assert ((T("") <= str0) != 0);
		assert ((T("") <  str0) == 0);
		assert ((T("") >= str0) != 0);
		assert ((T("") >  str0) == 0);

		assert ((T("") == str1) == 0);
		assert ((T("") != str1) != 0);
		assert ((T("") <= str1) != 0);
		assert ((T("") <  str1) != 0);
		assert ((T("") >= str1) == 0);
		assert ((T("") >  str1) == 0);

		assert (T("0123456789") == str1);
		assert ((T("0123456789") != str1) == 0);
		assert ((T("0123456789") <= str1) != 0);
		assert ((T("0123456789") <	str1) == 0);
		assert ((T("0123456789") >= str1) != 0);
		assert ((T("0123456789") >	str1) == 0);

		assert (str8.compare(str1) == 0);
		assert (str8.compare(str7) < 0);
		assert (str7.compare(str8) > 0);
		assert (str6.compare(str7) > 0);
		assert (str7.compare(str6) < 0);
		assert (str8.compare(T("0123456789")) == 0);
		assert (str8.compare(T("0123456789a")) < 0);
		assert (str8.compare(T("012345678")) > 0);
		//x assert (str4.compare(str0,2) == 0);
		//assert (str5.compare(str0,3,5) == 0);
		assert (str1.compare(3,5,str5) == 0);
		assert (str1.compare(3,4,str5) <  0);
		assert (str1.compare(3,6,str5) >  0);
		//x assert (str1.compare(3,5,str5,0) == 0);
		assert (str1.compare(3,5,str5,0,5) == 0);
		assert (str1.compare(3,5,str5,0,4) >  0);
		assert (str1.compare(3,4,str5,0,5) <  0);

		assert (str1.compare(3,5,T("3456789"),5) == 0);
		assert (str1.compare(3,5,T("3456789"),6) <	0);
		assert (str1.compare(3,5,T("3456789"),12) <  0);
		assert (str1.compare(3,5,T("3456789"),4) >	0);
		assert (str1.compare(2,5,T("3456789"),5) <	0);
		assert (str1.compare(2,5,T("3456789"),12) <  0);

		assert (str1.compare(8,8,str8, 8,8) == 0);
		assert (str1.compare(8,8,str8, 8,2) == 0);
		assert (str1.compare(8,8,str8, 8,1) >  0);
		assert (str1.compare(8,8,str8, 7,5) >  0);
		assert (str1.compare(8,8,str8, 9,2) <  0);

		assert (str1.compare(8,8,T("89")) == 0);
		assert (str1.compare(8,8,T("89"),2) == 0);
		assert (str1.compare(8,8,T("89abcd")) < 0);
		assert (str1.compare(8,8,T("89abcd"),6) < 0);
		assert (str1.compare(8,8,T("789abcd")) > 0);
		assert (str1.compare(8,8,T("789abcd"),7) > 0);
		assert (str1.compare(8,8,T("89abcd"), 2) == 0);
		assert (str1.compare(8,8,T("89"), 3) != 0);
		//x assert (str1.compare(8,8,T("89"), 5) == 0);	// T('\0')I[ƂĂ͈Ȃ͗l...

		assert (str1.compare(str0) > 0);
		assert (str0.compare(str7) < 0);
		assert (str0.compare(T("")) == 0);
		assert (str0.compare(T("012345")) < 0);

	  #if 0	// ItZbgʒusȏꍇ͗O𓊂ق悢?
		assert (str0.compare(3,5,str5) < 0);
		assert (str0.compare(3,5,str5,0,5) < 0);
		assert (str1.compare(3,5,str0,1,5) > 0);
		assert (str0.compare(3,5,str0,1,5) == 0);

		assert (str0.compare(3,5,T("3456789"),5) < 0);
		assert (str0.compare(3,5,T(""),5) == 0);
	  #endif
	}


	/// [], at, begin, end, rbegin, rend, swap gp
	void test_etc()
	{
		// ftHgERXgN^
		STRING	str0;
		assert (str0.empty());
		assert (str0.size() == 0);
		assert (str0.capacity() >= 0);
		assert (str0.max_size() >= str0.size());

		// 		U[u
		str0.reserve(100);
		assert (str0.empty());
		assert (str0.length() == 0);
		assert (str0.capacity() >= 100);
		assert (str0.max_size() >= str0.size());

		// 		TCY
		str0.resize(10, T(' '));
		assert (str0.empty() == false);
		assert (str0.size() == 10);
		assert (str0.capacity() >= 100);
		assert ( std::memcmp(&str0[0], T("          "), 10) == 0 );
		assert ( std::memcmp(str0.data() , T("          "), 10) == 0 );
		assert ( std::memcmp(str0.c_str(), T("          "), 11) == 0 );
		assert (str0.max_size() >= str0.size());

		//		at
	  #if !defined UNUSE_THROW
		try {
			for (int i = 0; i < 11; ++i)
				str0.at(i) = char(T('0')+i);
		} catch (...) {
			;
		}
		assert ( std::memcmp(str0.data() , T("0123456789"), 10) == 0 );
		assert ( std::memcmp(str0.c_str(), T("0123456789"), 11) == 0 );
	  #endif

		const STRING str1(T("0123456789"));
		//STRING	str1(10);
		assert (str1.empty() == false);
		assert (str1.size()   == 10);
		assert (str1.length() == 10);
		assert (str1.capacity() >= 10);
		assert (str1.max_size() >= str1.size());
		assert ( std::memcmp(str1.data() , T("0123456789"), 10) == 0 );
		assert ( std::memcmp(str1.c_str(), T("0123456789"), 11) == 0 );

	  #if !defined UNUSE_THROW
		//		[]	const at()
		try {
			for (int i = 0; i < 11; ++i) {
				int c = str1.at(9-i);
				str0[i] = c;
			}
		} catch (...) {
			;
		}
		assert ( std::memcmp(str0.data() , T("9876543210"), 10) == 0 );
		assert ( std::memcmp(str0.c_str(), T("9876543210"), 11) == 0 );
	  #endif

		//		const []
		for (int i = 0; i < 10; ++i)
			str0[i] = str1[i];
		assert ( std::memcmp(str1.data() , T("0123456789"), 10) == 0 );
		assert ( std::memcmp(str1.c_str(), T("0123456789"), 11) == 0 );

		// begin,end
		STRING	str2(str0.begin(), str0.end());
		assert ( str2 == T("0123456789") );

		// rbegin,rend
		STRING	str3(str0.rbegin(), str0.rend());
		assert ( str3 == T("9876543210") );

		// const begin, const end
		STRING	str4(str1.begin(), str1.end());
		assert ( str4 == T("0123456789") );

		// const rbegin, const rend
		STRING	str5(str1.rbegin(), str1.rend());
		assert ( str5 == T("9876543210") );

		str4.swap(str5);
		assert (str4 == T("9876543210"));
		assert (str5 == T("0123456789"));

		swap( str4, str5 );
		assert (str4 == T("0123456789"));
		assert (str5 == T("9876543210"));
	}


	/// copy
	void test_copy()
	{
		const STRING	str0(T("0123456789"));
		char buf[32];
		str0.copy(buf, 4);			assert(std::memcmp(buf, T("0123"), 4) == 0);
		str0.copy(buf, 10);			assert(std::memcmp(buf, T("0123456789"), 10) == 0);
		str0.copy(buf, 32);			assert(std::memcmp(buf, T("0123456789"), 10) == 0);
		str0.copy(buf, 4, 1);		assert(std::memcmp(buf, T("1234"), 4) == 0);
		str0.copy(buf,9, 1);		assert(std::memcmp(buf, T("123456789"), 9) == 0);
		str0.copy(buf, 32, 1);		assert(std::memcmp(buf, T("123456789"), 9) == 0);

		TEST_OUT_OF_RANGE( str0.copy(buf, 4, 100) );
		assert(std::memcmp(buf, T("123456789"), 9) == 0);
	}

	// swap
	void test_swap()
	{
		STRING str2 = T("abcdefghijklemopqrstuvwxyz");
		STRING str3 = T("0123456789ABCDEF");
		str2.swap(str3);
		assert (str2 == T("0123456789ABCDEF"));
		assert (str3 == T("abcdefghijklemopqrstuvwxyz"));

		swap( str2, str3 );
		assert (str2 == T("abcdefghijklemopqrstuvwxyz"));
		assert (str3 == T("0123456789ABCDEF"));
	}

	void test_substr()
	{
		const STRING str0(T("abcdefghijklmnopqrstuvwxyz"));
		STRING str1;
		str1 = str0.substr();			assert (str1 == T("abcdefghijklmnopqrstuvwxyz"));
		str1 = str0.substr(0, 5);		assert (str1 == T("abcde"));
		str1 = str0.substr(1,5);		assert (str1 == T("bcdef"));
		str1 = str0.substr(5);			assert (str1 == T("fghijklmnopqrstuvwxyz"));

		str1 = str0.substr(25);			assert (str1 == T("z"));

		str1 = str0.substr(25, 10);		assert (str1 == T("z"));

		str1 = str0.substr(26);			assert (str1 == T(""));

		TEST_OUT_OF_RANGE( str1.substr(27) );
		TEST_OUT_OF_RANGE( str1.substr(27, 10) );
	}


	/// ( =, assign )
	void test_assign()
	{
		const STRING	str0(T("0123456789"));

		STRING str1;
		str1 = str0;
		assert( str1 == str0 );
		assert( str1.size() == 10 && std::memcmp(str1.c_str(), str0.c_str(), 10) == 0);

		str1 = str1;
		assert( str1 == str0 );
		assert( str1.size() == 10 && std::memcmp(str1.c_str(), str0.c_str(), 10) == 0);

		str1 = T("abcdefg");
		assert( str1 == T("abcdefg") );

		str1 = T('z');
		assert( str1 == T("z") );

		str1.assign(10, T('@'));
		assert( str1 == T("@@@@@@@@@@") );

		str1.assign(T("0123456789"));
		assert( str1 == T("0123456789") );

		str1.assign(T("0123456789"), 5);
		assert( str1 == T("01234") );

		str1.assign(str0);
		assert( str1 == T("0123456789") );

		TEST_OUT_OF_RANGE( str1.assign(str0, 100, 5) );

		str1.assign(str0, 3,5);
		assert( str1 == T("34567") );

		str1.assign(str0.rbegin(), str0.rend());
		assert( str1 == T("9876543210") );

		str1.assign(str0.end(), str0.end());
		assert( str1 == T("") );

	  #if 0
		str1.assign(str0, 4);
		assert( str1 == T("456789") );
	  #endif

	}

	/// ǉ
	void test_append()
	{
		const STRING	str0(T("0123456789"));

		STRING str1;
		str1 += T("");
		assert( str1.empty() );
		str1 += str0;
		assert( str1 == str0 );
		assert( str1.size() == 10 && std::memcmp(str1.c_str(), str0.c_str(), 10) == 0);

		str1 += T("abcdefg");
		assert( str1 == T("0123456789abcdefg") );

		str1 += T('_');
		assert( str1 == T("0123456789abcdefg_") );

		str1.append(T("testtest"));
		assert( str1 == T("0123456789abcdefg_testtest") );

		str1.append(T("0123456789"), 5);
		assert( str1 == T("0123456789abcdefg_testtest01234") );

		str1.append(10, T('@'));
		assert( str1 == T("0123456789abcdefg_testtest01234@@@@@@@@@@") );

		str1.append(str0);
		assert( str1 == T("0123456789abcdefg_testtest01234@@@@@@@@@@0123456789") );

		TEST_OUT_OF_RANGE( str1.append(str0, 133, 5) );

		str1.append(str0, 3,5);
		assert( str1 == T("0123456789abcdefg_testtest01234@@@@@@@@@@012345678934567") );

		str1.append(str0.rbegin(), str0.rend());
		assert( str1 == T("0123456789abcdefg_testtest01234@@@@@@@@@@0123456789345679876543210") );

		str1.append(str0.rend(), str0.rend());
		assert( str1 == T("0123456789abcdefg_testtest01234@@@@@@@@@@0123456789345679876543210") );

	  #if 0
		str1.append(str0, 4);
		assert( str1 == T("0123456789abcdefg_testtest01234@@@@@@@@@@0123456789345679876543210456789") );
	  #endif
	}

	void test_join()
	{
		const STRING	str0;
		const STRING	str1 = T("0123456789");
		const STRING	str2 = T("abcde");
		STRING			str3;
		str3 = str0 + str1;
		assert( str3 == T("0123456789") );
		str3 = str1 + str0;
		assert( str3 == T("0123456789") );

		str3 = str2 + str1;
		assert( str3 == T("abcde0123456789") );
		str3 = str1 + str2;
		assert( str3 == T("0123456789abcde") );

		str3 = T("") + str0;
		assert( str3.empty() );
		str3 = str0 + T("");
		assert( str3.empty() );

		str3 = T("") + str1;
		assert( str3 == T("0123456789") );
		str3 = str1 + T("");
		assert( str3 == T("0123456789") );

		str3 = T("@") + str1;
		assert( str3 == T("@0123456789") );
		str3 = str1 + T("@");
		assert( str3 == T("0123456789@") );

		str3 = T("string") + str1;
		assert( str3 == T("string0123456789") );
		str3 = str1 + T("string");
		assert( str3 == T("0123456789string") );

		str3 = T('@') + str0;
		assert( str3 == T("@") );
		str3 = str0 + T('@');
		assert( str3 == T("@") );

		str3 = T('@') + str1;
		assert( str3 == T("@0123456789") );
		str3 = str1 + T('@');
		assert( str3 == T("0123456789@") );
	}

	/// 폜
	void test_erase()
	{
		STRING	str0 = T("0123456789");
		str0.erase(9);
		assert( str0 == T("012345678") );
		str0.erase(8,1);
		assert( str0 == T("01234567") );
		str0.erase(6);
		assert( str0 == T("012345") );
		str0.erase(4,3);
		assert( str0 == T("0123") );
		str0.erase(1,2);
		assert( str0 == T("03") );
		str0.erase(0,1);
		assert( str0 == T("3") );
		str0.erase(0);
		assert( str0 == T("") );

		str0 = T("0123456789");

#if _MSC_VER < 1400
		TEST_OUT_OF_RANGE( str0.erase(str0.end()+1) );
#endif
		str0.erase(str0.end());
		assert( str0 == T("0123456789") );

		str0.erase(ind2ite(str0,4), ind2ite(str0,7));
		assert( str0 == T("0123789") );

		str0.erase(ind2ite(str0,3));
		assert( str0 == T("012789") );

		str0.erase(str0.begin(), str0.end());
		assert( str0 == T("") );

	#if _MSC_VER < 1400
		str0 = T("0123456789");
		str0.erase( str0.begin(), str0.end()+100);
		assert( str0 == T("") );

		str0 = T("0123456789");
		TEST_OUT_OF_RANGE( str0.erase( str0.begin()-10, str0.end() ) );
		assert( str0 == T("0123456789") );

		STRING str1(T("abcdefg"));
		str0 = T("0123456789");
		TEST_OUT_OF_RANGE( str0.erase( str1.begin(), str1.end() ) );
		assert( str0 == T("0123456789") );
	#endif
	}

	/// }
	void test_insert()
	{
		STRING	str0;

		TEST_OUT_OF_RANGE( str0.insert(1, T("236")) );
		str0.insert(0, T("236"));
		assert( str0 == T("236") );

		TEST_OUT_OF_RANGE( str0.insert(6, T("01"),1) );
		str0.insert(0, T("01"),1);
		assert( str0 == T("0236") );

		TEST_OUT_OF_RANGE( str0.insert(unsigned(-1), 1, T('1')) );
		str0.insert(1, 1, T('1'));
		assert( str0 == T("01236") );

		TEST_OUT_OF_RANGE( str0.insert(40, T("45")) );
		str0.insert(4, T("45"));
		assert( str0 == T("0123456") );

		str0.insert(7, T("789"));
		assert( str0 == T("0123456789") );

		str0.insert(10, T("@#%$!"),5);
		assert( str0 == T("0123456789@#%$!"));

		STRING	str1;

		TEST_OUT_OF_RANGE( str1.insert(100, str0, 10, 3) );
		str1.insert(0, str0, 10, 3);
		assert( str1 == T("@#%") );

		TEST_OUT_OF_RANGE( str1.insert(100, str0, 13, STRING::npos) );
		str1.insert(3, str0, 13, STRING::npos);
		assert( str1 == T("@#%$!") );

		str1.insert(5, str0, 0, 10);
		assert( str1 == T("@#%$!0123456789") );

		TEST_OUT_OF_RANGE( str1.insert(str1.size()+1, str0) );
		str1.insert(str1.size(), str0);
		assert( str1 == T("@#%$!01234567890123456789@#%$!") );

	#if _MSC_VER < 1400
		TEST_OUT_OF_RANGE( str1.insert(str0.begin(), T('-')) );
		str1.insert(ind2ite(str1,5), T('-'));
		assert( str1 == T("@#%$!-01234567890123456789@#%$!") );

		TEST_OUT_OF_RANGE( str1.insert(str0.begin(), 4, T('+')) );
		str1.insert(ind2ite(str1,6), 4, T('+'));
		assert( str1 == T("@#%$!-++++01234567890123456789@#%$!") );

		TEST_OUT_OF_RANGE( str1.insert(ind2ite(str0,5), str0.rbegin(), str0.rend()) );
		str1.insert(ind2ite(str1,5), str0.rbegin(), str0.rend());
		assert( str1 == T("@#%$!!$%#@9876543210-++++01234567890123456789@#%$!") );
	#endif
	}


	// u
	void test_replace()
	{
		// }
		STRING	str0;

		TEST_OUT_OF_RANGE( str0.replace(1, 5, T("236")) );
		str0.replace(0,1, T("236"));
		assert( str0 == T("236") );

		TEST_OUT_OF_RANGE( str0.replace(6,1, T("01"),1) );
		str0.replace(0,0, T("01"),1);
		assert( str0 == T("0236") );

		TEST_OUT_OF_RANGE( str0.replace(unsigned(-1),1, 1, T('1')) );
		str0.replace(1,0, 1, T('1'));
		assert( str0 == T("01236") );

		TEST_OUT_OF_RANGE( str0.replace(40,0, T("45")) );
		str0.replace(4,0, T("45"));
		assert( str0 == T("0123456") );

		str0.replace(7,0, T("789"));
		assert( str0 == T("0123456789") );

		str0.replace(10,0, T("@#%$!"),5);
		assert( str0 == T("0123456789@#%$!"));

		STRING	str1;

		TEST_OUT_OF_RANGE( str1.replace(100,0, str0, 10, 3) );
		str1.replace(0,0, str0, 10, 3);
		assert( str1 == T("@#%") );

		TEST_OUT_OF_RANGE( str1.replace(100,0, str0, 13, STRING::npos) );
		str1.replace(3,0, str0, 13, STRING::npos);
		assert( str1 == T("@#%$!") );

		str1.replace(5,0, str0, 0, 10);
		assert( str1 == T("@#%$!0123456789") );

		TEST_OUT_OF_RANGE( str1.replace(str1.size()+1,0, str0) );
		str1.replace(str1.size(),0, str0);
		assert( str1 == T("@#%$!01234567890123456789@#%$!") );

	#if _MSC_VER < 1400
		TEST_OUT_OF_RANGE( str1.replace(ind2ite(str0,0),ind2ite(str0,1), 1, T('-')) );
		str1.replace(ind2ite(str1,5),ind2ite(str1,5), 1,T('-'));
		assert( str1 == T("@#%$!-01234567890123456789@#%$!") );

		TEST_OUT_OF_RANGE( str1.replace(ind2ite(str0,0),ind2ite(str0,0), 4, T('+')) );
		str1.replace(ind2ite(str1,6),ind2ite(str1,6), 4, T('+'));
		assert( str1 == T("@#%$!-++++01234567890123456789@#%$!") );

		TEST_OUT_OF_RANGE( str1.replace(ind2ite(str0,5),ind2ite(str0,5), str0.rbegin(), str0.rend()) );
		str1.replace(ind2ite(str1,5),ind2ite(str1,5), str0.rbegin(), str0.rend());
		assert( str1 == T("@#%$!!$%#@9876543210-++++01234567890123456789@#%$!") );
	#endif


		// 폜
		STRING str2;
		str0 = T("0123456789");
		str0.replace(9,str0.npos, T(""));
		assert( str0 == T("012345678") );
		str0.replace(8,1, str2);
		assert( str0 == T("01234567") );
		str0.replace(6,str0.npos, str2, 0, 1);
		assert( str0 == T("012345") );
		str0.replace(4,3, T(""));
		assert( str0 == T("0123") );
		str0.replace(1,2, str2);
		assert( str0 == T("03") );
		str0.replace(0,1, str2, 0, 0);
		assert( str0 == T("3") );
		str0.replace(0,1, T(""), 0);
		assert( str0 == T("") );

		str0 = T("0123456789");

	#if _MSC_VER < 1400
		TEST_OUT_OF_RANGE( str0.replace(str0.end()+1, str0.end()+1, T("")) );
	#endif
		str0.replace(str0.end(), str0.end(), T(""));
		assert( str0 == T("0123456789") );

		str0.replace(ind2ite(str0,4), ind2ite(str0,7), str2);
		assert( str0 == T("0123789") );

		str0.replace(ind2ite(str0,3), ind2ite(str0,4), str2.begin(), str2.end());
		assert( str0 == T("012789") );

		str0.replace(str0.begin(), str0.end(), str2);
		assert( str0 == T("") );

	#if _MSC_VER < 1400
		str0 = T("0123456789");
		str0.replace( str0.begin(), str0.end()+100, str2);
		assert( str0 == T("") );

		str0 = T("0123456789");
		TEST_OUT_OF_RANGE( str0.replace( str0.begin()-10, str0.end(), T("") ) );
		TEST_OUT_OF_RANGE( str0.replace( str0.begin()-10, str0.end()+10, T("") ) );
		assert( str0 == T("0123456789") );

		str1 = T("abcdefg");
		str0 = T("0123456789");
		TEST_OUT_OF_RANGE( str0.replace( str1.begin(), str1.end(), T("") ) );
		TEST_OUT_OF_RANGE( str0.replace( str1.begin(), str1.end(), str2.begin(), str2.end() ) );
		assert( str0 == T("0123456789") );
	#endif

		// u
		str0.replace(str0.begin(), str0.end(), T(""));
		assert( str0 == T("") );
		str1.replace(str1.begin(), str1.end(), T("ABCDEFGHIJ"));
		assert( str1 == T("ABCDEFGHIJ") );

		str0.replace(0,0, T("000222"));
		assert( str0 == T("000222") );
		str0.replace(1,4, T("1"));
		assert( str0 == T("012") );
		str0.replace(str0.end(),str0.end(), T("3456"), 4);
		assert( str0 == T("0123456") );
		str0.replace(5,5, 10, T('-'));
		assert( str0 == T("01234----------") );
		str0.replace(ind2ite(str0,10), ind2ite(str0,12), 5, T('+'));
		assert( str0 == T("01234-----+++++---") );
		str0.replace(ind2ite(str0,8), ind2ite(str0,10), T("89abcdef"), 5);
		assert( str0 == T("01234---89abc+++++---") );
		str0.replace(0, 5, T("abcdefghij"), 5);
		assert( str0 == T("abcde---89abc+++++---") );
		str0.replace(0, 5, T("0123456789"));
		assert( str0 == T("0123456789---89abc+++++---") );
		str0.replace(0, 15, str1);
		assert( str0 == T("ABCDEFGHIJabc+++++---") );

		str0.replace(0, str0.max_size(), T(""));
		assert( str0 == T("") );
		str0.replace(0,0, T("0123456789"));
		assert( str0 == T("0123456789") );
		str0.replace(5,1, T("G"));
		assert( str0 == T("01234G6789") );
		str0.replace(5,1, 1, T('g'));
		assert( str0 == T("01234g6789") );
		str0.replace(5,1, 5, T('v'));
		assert( str0 == T("01234vvvvv6789") );
		str0.replace(5,5, T("5#"));
		assert( str0 == T("012345#6789") );
		str0.replace(6,1, T("abcdefghij"), 6);
		assert( str0 == T("012345abcdef6789") );
		str0.replace(6,6, str1, 1,3);
		assert( str0 == T("012345BCD6789") );
		str0.replace(6,3, str1);
		assert( str0 == T("012345ABCDEFGHIJ6789") );
		str0.replace(6,10, str1, 0,0);
		assert( str0 == T("0123456789") );
		str0.replace(7,0, str1, 0,10);
		assert( str0 == T("0123456ABCDEFGHIJ789") );
		str0.replace(7,10, str2, 0,10);
		assert( str0 == T("0123456789") );

		str0.replace(str0.begin(), str0.end(), T(""), size_t(0U));
		assert( str0 == T("") );
		str0.replace(str0.begin(), str0.begin(), T("0123456789"));
		assert( str0 == T("0123456789") );
		str0.replace(ind2ite(str0,5), ind2ite(str0,6), T("G"));
		assert( str0 == T("01234G6789") );
		str0.replace(ind2ite(str0,5), ind2ite(str0,6), 1, T('g'));
		assert( str0 == T("01234g6789") );
		str0.replace(ind2ite(str0,5), ind2ite(str0,6), 5, T('v'));
		assert( str0 == T("01234vvvvv6789") );
		str0.replace(ind2ite(str0,5), ind2ite(str0,10), T("5#"));
		assert( str0 == T("012345#6789") );
		str0.replace(ind2ite(str0,6), ind2ite(str0,7), T("abcdefghij"), 6);
		assert( str0 == T("012345abcdef6789") );
		str0.replace(ind2ite(str0,6), ind2ite(str0,12), ind2ite(str1,1), ind2ite(str1,4));
		assert( str0 == T("012345BCD6789") );
		str0.replace(ind2ite(str0,6), ind2ite(str0,9), str1);
		assert( str0 == T("012345ABCDEFGHIJ6789") );
		str0.replace(ind2ite(str0,6), ind2ite(str0,16), ind2ite(str1,0), ind2ite(str1,0));
		assert( str0 == T("0123456789") );
		str0.replace(ind2ite(str0,7), ind2ite(str0,7), ind2ite(str1,0), ind2ite(str1,10));
		assert( str0 == T("0123456ABCDEFGHIJ789") );
		str0.replace(ind2ite(str0,7), ind2ite(str0,17), str2);
		assert( str0 == T("0123456789") );

	}

	void test_find()
	{
		std::size_t 	n;
		const STRING	text0;
		n = text0.find(T('0'));				assert( n == text0.npos );
		n = text0.find(T('\0'));			assert( n == text0.npos );
		n = text0.find(T("0"));				assert( n == text0.npos );
		n = text0.find(T('0'), 100);		assert( n == text0.npos );
		n = text0.find(T('\0'),100);		assert( n == text0.npos );
		n = text0.find(T("0"), 100);		assert( n == text0.npos );

	  #if 1
		n = text0.find(T(""));				assert( n == 0 );
		n = text0.find(T("") , 100);		assert( n == text0.npos );
	  #endif

		const STRING text =  T("0123456789"
							   "abcdefghij"
							   "klmnopqrst"
							   "uvwxyz####");

		// 
		n = text.find(T('0'));				assert( n == 0 );
		n = text.find(T('7'));				assert( n == 7 );
		n = text.find(T("0"));				assert( n == 0 );
		n = text.find(T("89"));				assert( n == 8 );
		n = text.find(T("klmno"));			assert( n == 20 );
		n = text.find(T("end"));			assert( n == text.npos );
		n = text.find(T('\0'));				assert( n == text.npos );
		n = text.find(T(""));				assert( n == 0 );
		//n = text.find((char*)0);			assert( n == 0 );

		n = text.find(T('0'), 1);			assert( n == text.npos );
		n = text.find(T('7'), 1);			assert( n == 7 );
		n = text.find(T("0"), 1);			assert( n == text.npos );
		n = text.find(T("67"), 8);			assert( n == text.npos );
		n = text.find(T("78"), 8);			assert( n == text.npos );
		n = text.find(T("89"), 8);			assert( n == 8 );
		n = text.find(T("klmno"), 20);		assert( n == 20 );
		n = text.find(T("end"), 1);			assert( n == text.npos );
		n = text.find(T('\0'), 8);			assert( n == text.npos );
		n = text.find(T(""), 8);			assert( n == 8 );

		n = text.find(STRING(T("0")));				assert( n == 0 );
		n = text.find(STRING(T("89")));				assert( n == 8 );
		n = text.find(STRING(T("klmno")));			assert( n == 20 );
		n = text.find(STRING(T("end")));			assert( n == text.npos );
		n = text.find(STRING(T("0")), 1);			assert( n == text.npos );
		n = text.find(STRING(T("67")), 8);			assert( n == text.npos );
		n = text.find(STRING(T("78")), 8);			assert( n == text.npos );
		n = text.find(STRING(T("89")), 8);			assert( n == 8 );
		n = text.find(STRING(T("klmno")), 20);		assert( n == 20 );
		n = text.find(STRING(T("klmno")), 21);		assert( n == text.npos );
		n = text.find(STRING(T("end")), 1);			assert( n == text.npos );
	  #if 0 //ndef _MSC_VER
		n = text.find(STRING(T('\0')), 1);			assert( n == text.npos );
	  #endif
		n = text.find(STRING(T("")), 8);			assert( n == 8 );

		// -----
		STRING text2(text);
		text2 += text;

		n = text2.find(T('0'));				assert( n == 0 );
		n = text2.find(T('7'));				assert( n == 7 );
		n = text2.find(T("0"));				assert( n == 0 );
		n = text2.find(T("89"));			assert( n == 8 );
		n = text2.find(T("end"));			assert( n == text2.npos );

		n = text2.find(T('0'), 1);			assert( n == 40 );
		n = text2.find(T('7'), 1);			assert( n == 7 );
		n = text2.find(T("0"), 1);			assert( n == 40 );
		n = text2.find(T("78"), 8);			assert( n == 47 );
		n = text2.find(T("89"), 8);			assert( n == 8 );
		n = text2.find(T("end"), 1);		assert( n == text2.npos );

		n = text2.find(STRING(T("0")));				assert( n == 0 );
		n = text2.find(STRING(T("89")));			assert( n == 8 );
		n = text2.find(STRING(T("end")));			assert( n == text2.npos );
		n = text2.find(STRING(T("0")), 1);			assert( n == 40 );
		n = text2.find(STRING(T("78")), 8);			assert( n == 47 );
		n = text2.find(STRING(T("89")), 8);			assert( n == 8 );
		n = text2.find(STRING(T("end")), 1);		assert( n == text2.npos );
	}


	void test_rfind()
	{
		STRING	text;
		std::size_t n;

		n = text.rfind(T('0'));				assert( n == text.npos );
		n = text.rfind(T('\0'));			assert( n == text.npos );
		n = text.rfind(T("0"));				assert( n == text.npos );
		n = text.rfind(T('0'), 100);		assert( n == text.npos );
		n = text.rfind(T('\0'),100);		assert( n == text.npos );
		n = text.rfind(T("0"), 100);		assert( n == text.npos );

	  #if 1
		n = text.rfind(T(""));				assert( n == 0 );
		n = text.rfind(T("") , 100);		assert( n == 0 );
	  #endif

		text =	T("0123456789")
				T("abcdefghij")
				T("klmnopqrst")
				T("uvwxyz####");

		n = text.rfind(T('0'));				assert( n == 0 );
		n = text.rfind(T('7'));				assert( n == 7 );
		n = text.rfind(T("0"));				assert( n == 0 );
		n = text.rfind(T("89"));			assert( n == 8 );
		n = text.rfind(T("klmno"));			assert( n == 20 );
		n = text.rfind(T("end"));			assert( n == text.npos );

		n = text.rfind(T('0'), 6);			assert( n == 0 );
		n = text.rfind(T('7'), 6);			assert( n == text.npos );
		n = text.rfind(T("0"), 8);			assert( n == 0 );
		n = text.rfind(T("67"), 7 );		assert( n == 6 );
		n = text.rfind(T("78"), 7 );		assert( n == 7 );
		n = text.rfind(T("89"), 7 );		assert( n == text.npos );
		n = text.rfind(T("klmno"), 30 );	assert( n == 20 );
		n = text.rfind(T("end"), 38 );		assert( n == text.npos );

		n = text.rfind(STRING(T("0")));				assert( n == 0 );
		n = text.rfind(STRING(T("89")));			assert( n == 8 );
		n = text.rfind(STRING(T("klmno")));			assert( n == 20 );
		n = text.rfind(STRING(T("end")));			assert( n == text.npos );
		n = text.rfind(STRING(T("0")), 8);			assert( n == 0 );
		n = text.rfind(STRING(T("67")), 7 );		assert( n == 6 );
		n = text.rfind(STRING(T("78")), 7 );		assert( n == 7 );
		n = text.rfind(STRING(T("89")), 7 );		assert( n == text.npos );
		n = text.rfind(STRING(T("klmno")), 30 );	assert( n == 20 );
		n = text.rfind(STRING(T("end")), 38 );		assert( n == text.npos );

		// -----
		text += STRING(text);

		n = text.rfind(T('0'));				assert( n == 40 );
		n = text.rfind(T('7'));				assert( n == 47 );
		n = text.rfind(T("0"));				assert( n == 40 );
		n = text.rfind(T("89"));			assert( n == 48 );
		n = text.rfind(T("klmno"));			assert( n == 60 );
		n = text.rfind(T("end"));			assert( n == text.npos );

		n = text.rfind(T('0'), 46);			assert( n == 40 );
		n = text.rfind(T('7'), 46);			assert( n == 7 );
		n = text.rfind(T("0"), 48);			assert( n == 40 );
		n = text.rfind(T("67"), 47 );		assert( n == 46 );
		n = text.rfind(T("78"), 47 );		assert( n == 47 );
		n = text.rfind(T("89"), 47 );		assert( n == 8	);
		n = text.rfind(T("klmno"), 30 );	assert( n == 20 );
		n = text.rfind(T("end"), 79 );		assert( n == text.npos );

		n = text.rfind(STRING(T("0")));				assert( n == 40 );
		n = text.rfind(STRING(T("89")));			assert( n == 48 );
		n = text.rfind(STRING(T("klmno")));			assert( n == 60 );
		n = text.rfind(STRING(T("end")));			assert( n == text.npos );
		n = text.rfind(STRING(T("0")), 48);			assert( n == 40 );
		n = text.rfind(STRING(T("67")), 47 );		assert( n == 46 );
		n = text.rfind(STRING(T("78")), 47 );		assert( n == 47 );
		n = text.rfind(STRING(T("89")), 47 );		assert( n == 8	);
		n = text.rfind(STRING(T("klmno")), 30);		assert( n == 20 );
		n = text.rfind(STRING(T("end")), 79 );		assert( n == text.npos );
	}

	void test_find_first_of()
	{
		STRING	text =	T("0123456789")
						T("abcdefghij")
						T("klmnopqrst")
						T("uvwxyz###.");
		std::size_t n;

		n = text.find_first_of(T('0'));				assert( n == 0 );
		n = text.find_first_of(T('a'));				assert( n == 10 );
		n = text.find_first_of(T('.'));				assert( n == 39 );
		n = text.find_first_of(T('\0'));			assert( n == text.npos );
		n = text.find_first_of(T('?'));				assert( n == text.npos );

		n = text.find_first_of(T('0'),10);			assert( n == text.npos );
		n = text.find_first_of(T('a'),10);			assert( n == 10 );
		n = text.find_first_of(T('.'),10);			assert( n == 39 );
		n = text.find_first_of(T('\0'),10);			assert( n == text.npos );
		n = text.find_first_of(T('?'),10);			assert( n == text.npos );
		n = text.find_first_of(T('0'),text.npos);	assert( n == text.npos );

		n = text.find_first_of(T('0'),100);			assert( n == text.npos );
		n = text.find_first_of(T('a'),100);			assert( n == text.npos );
		n = text.find_first_of(T('.'),100);			assert( n == text.npos );
		n = text.find_first_of(T('\0'),100);		assert( n == text.npos );
		n = text.find_first_of(T('?'),100);			assert( n == text.npos );

		n = text.find_first_of(T("0"));				assert( n == 0 );
		n = text.find_first_of(T("a"));				assert( n == 10 );
		n = text.find_first_of(T("."));				assert( n == 39 );
		n = text.find_first_of(T("?"));				assert( n == text.npos );

		n = text.find_first_of(T("0"),10);			assert( n == text.npos );
		n = text.find_first_of(T("a"),10);			assert( n == 10 );
		n = text.find_first_of(T("."),10);			assert( n == 39 );
		n = text.find_first_of(T("?"),10);			assert( n == text.npos );
		n = text.find_first_of(T("0"),text.npos);	assert( n == text.npos );

		n = text.find_first_of(T("0"),100);			assert( n == text.npos );
		n = text.find_first_of(T("a"),100);			assert( n == text.npos );
		n = text.find_first_of(T("."),100);			assert( n == text.npos );
		n = text.find_first_of(T("?"),100);			assert( n == text.npos );

		n = text.find_first_of(T("0123456789"));	assert( n == 0 );
		n = text.find_first_of(T("badcfe"));		assert( n == 10 );
		n = text.find_first_of(T(".#"));			assert( n == 36 );

		n = text.find_first_of(T("0123456789"),5);	assert( n == 5 );
		n = text.find_first_of(T("badcfe"), 5);		assert( n == 10 );
		n = text.find_first_of(T("0123456789"),10);	assert( n == text.npos );
		n = text.find_first_of(T("badcfe"), 10);	assert( n == 10 );
		n = text.find_first_of(T(".#"), 10);		assert( n == 36 );
		n = text.find_first_of(T("a0"),text.npos);	assert( n == text.npos );


		n = text.find_first_of(STRING(T("0")));			assert( n == 0 );
		n = text.find_first_of(STRING(T("a")));			assert( n == 10 );
		n = text.find_first_of(STRING(T(".")));			assert( n == 39 );
		n = text.find_first_of(STRING(T("?")));			assert( n == text.npos );

		n = text.find_first_of(STRING(T("0")),10);			assert( n == text.npos );
		n = text.find_first_of(STRING(T("a")),10);			assert( n == 10 );
		n = text.find_first_of(STRING(T(".")),10);			assert( n == 39 );
		n = text.find_first_of(STRING(T("?")),10);			assert( n == text.npos );
		n = text.find_first_of(STRING(T("0")),text.npos);	assert( n == text.npos );

		n = text.find_first_of(STRING(T("0123456789")));	assert( n == 0 );
		n = text.find_first_of(STRING(T("badcfe")));		assert( n == 10 );
		n = text.find_first_of(STRING(T(".#")));			assert( n == 36 );

		n = text.find_first_of(STRING(T("0123456789")),5);	assert( n == 5 );
		n = text.find_first_of(STRING(T("badcfe")), 5);		assert( n == 10 );
		n = text.find_first_of(STRING(T("0123456789")),10);	assert( n == text.npos );
		n = text.find_first_of(STRING(T("badcfe")), 10);	assert( n == 10 );
		n = text.find_first_of(STRING(T(".#")), 10);		assert( n == 36 );
		n = text.find_first_of(STRING(T("a0")),text.npos);	assert( n == text.npos );
		n = text.find_first_of(STRING(T("")),text.npos);	assert( n == text.npos );


		n = text.find_first_of(T("\0"));					assert( n == text.npos );
		n = text.find_first_of(T("\0"),10);					assert( n == text.npos );
		n = text.find_first_of(T("\0"),100);				assert( n == text.npos );
		n = text.find_first_of(STRING(T("")));				assert( n == text.npos );
		n = text.find_first_of(STRING(T("")), 10);			assert( n == text.npos );
		n = text.find_first_of(STRING(T("\0")));			assert( n == text.npos );
		n = text.find_first_of(STRING(T("\0")),10);			assert( n == text.npos );


		text += STRING(text);

		n = text.find_first_of(T('0'));				assert( n == 0 );
		n = text.find_first_of(T('a'));				assert( n == 10 );
		n = text.find_first_of(T('.'));				assert( n == 39 );
		n = text.find_first_of(T('\0'));			assert( n == text.npos );
		n = text.find_first_of(T('?'));				assert( n == text.npos );

		n = text.find_first_of(T('0'),10);			assert( n == 40 );
		n = text.find_first_of(T('a'),10);			assert( n == 10 );
		n = text.find_first_of(T('.'),10);			assert( n == 39 );
		n = text.find_first_of(T('\0'),10);			assert( n == text.npos );
		n = text.find_first_of(T('?'),10);			assert( n == text.npos );
		n = text.find_first_of(T('0'),text.npos);	assert( n == text.npos );

		n = text.find_first_of(T("0"));				assert( n == 0 );
		n = text.find_first_of(T("a"));				assert( n == 10 );
		n = text.find_first_of(T("."));				assert( n == 39 );
		n = text.find_first_of(T("\0"));			assert( n == text.npos );
		n = text.find_first_of(T("?"));				assert( n == text.npos );

		n = text.find_first_of(T("0"),10);			assert( n == 40 );
		n = text.find_first_of(T("a"),10);			assert( n == 10 );
		n = text.find_first_of(T("."),10);			assert( n == 39 );
		n = text.find_first_of(T("\0"),10);			assert( n == text.npos );
		n = text.find_first_of(T("?"),10);			assert( n == text.npos );
		n = text.find_first_of(T("0"),text.npos);	assert( n == text.npos );

		n = text.find_first_of(T("0123456789"));	assert( n == 0 );
		n = text.find_first_of(T("badcfe"));		assert( n == 10 );
		n = text.find_first_of(T(".#"));			assert( n == 36 );

		n = text.find_first_of(T("0123456789"),5);	assert( n == 5 );
		n = text.find_first_of(T("badcfe"), 5);		assert( n == 10 );
		n = text.find_first_of(T("0123456789"),10);	assert( n == 40 );
		n = text.find_first_of(T("badcfe"), 10);	assert( n == 10 );
		n = text.find_first_of(T(".#"), 10);		assert( n == 36 );
		n = text.find_first_of(T("a0"),text.npos);	assert( n == text.npos );

		n = text.find_first_of(STRING(T("0")));			assert( n == 0 );
		n = text.find_first_of(STRING(T("a")));			assert( n == 10 );
		n = text.find_first_of(STRING(T(".")));			assert( n == 39 );
		n = text.find_first_of(STRING(T("\0")));		assert( n == text.npos );
		n = text.find_first_of(STRING(T("?")));			assert( n == text.npos );

		n = text.find_first_of(STRING(T("0")),10);			assert( n == 40 );
		n = text.find_first_of(STRING(T("a")),10);			assert( n == 10 );
		n = text.find_first_of(STRING(T(".")),10);			assert( n == 39 );
		n = text.find_first_of(STRING(T("\0")),10);			assert( n == text.npos );
		n = text.find_first_of(STRING(T("?")),10);			assert( n == text.npos );
		n = text.find_first_of(STRING(T("0")),text.npos);	assert( n == text.npos );

		n = text.find_first_of(STRING(T("0123456789")));	assert( n == 0 );
		n = text.find_first_of(STRING(T("badcfe")));		assert( n == 10 );
		n = text.find_first_of(STRING(T(".#")));			assert( n == 36 );

		n = text.find_first_of(STRING(T("0123456789")),5);	assert( n == 5 );
		n = text.find_first_of(STRING(T("badcfe")), 5);		assert( n == 10 );
		n = text.find_first_of(STRING(T("0123456789")),10);	assert( n == 40 );
		n = text.find_first_of(STRING(T("badcfe")), 10);	assert( n == 10 );
		n = text.find_first_of(STRING(T(".#")), 10);		assert( n == 36 );
		n = text.find_first_of(STRING(T("a0")),text.npos);	assert( n == text.npos );
	}

	void test_find_last_of()
	{
		STRING	text =	T("0123456789")
						T("abcdefghij")
						T("klmnopqrst")
						T("uvwxyz###.");
		std::size_t n;

		n = text.find_last_of(T('0'));				assert( n == 0 );
		n = text.find_last_of(T('a'));				assert( n == 10 );
		n = text.find_last_of(T('.'));				assert( n == 39 );
	  #ifdef _MSC_VER
		n = text.find_last_of(T('\0'));				assert( n == text.npos );
	  #endif
		n = text.find_last_of(T('?'));				assert( n == text.npos );

		n = text.find_last_of(T('0'),10);			assert( n == 0 );
		n = text.find_last_of(T('a'),10);			assert( n == 10 );
		n = text.find_last_of(T('.'),10);			assert( n == text.npos );
	  #ifdef _MSC_VER
		n = text.find_last_of(T('\0'),10);			assert( n == text.npos );
	  #endif
		n = text.find_last_of(T('?'),10);			assert( n == text.npos );
		n = text.find_first_of(T('0'),0);			assert( n == 0 );

		n = text.find_last_of(T("0"));				assert( n == 0 );
		n = text.find_last_of(T("a"));				assert( n == 10);
		n = text.find_last_of(T("."));				assert( n == 39);
	  #ifdef _MSC_VER
		n = text.find_last_of(T("\0"));				assert( n == text.npos );
	  #endif
		n = text.find_last_of(T("?"));				assert( n == text.npos );

		n = text.find_last_of(T("0"),10);			assert( n == 0 );
		n = text.find_last_of(T("a"),10);			assert( n == 10 );
		n = text.find_last_of(T("."),10);			assert( n == text.npos );
	  #ifdef _MSC_VER
		n = text.find_last_of(T("\0"),10);			assert( n == text.npos );
	  #endif
		n = text.find_last_of(T("?"),10);			assert( n == text.npos );
		n = text.find_first_of(T("0"),0);			assert( n == 0 );

		n = text.find_last_of(T("0123456789"));		assert( n == 9	);
		n = text.find_last_of(T("badcfe"));			assert( n == 15 );
		n = text.find_last_of(T(".#"));				assert( n == 39 );

		n = text.find_last_of(T("0123456789"),5);	assert( n == 5 );
		n = text.find_last_of(T("badcfe"), 5);		assert( n == text.npos);
		n = text.find_last_of(T("0123456789"), 9);	assert( n == 9 );
		n = text.find_last_of(T("badcfe"), 9);		assert( n == text.npos );
		n = text.find_last_of(T(".#"), 38 );		assert( n == 38 );
		n = text.find_first_of(T("a0"),0);			assert( n == 0 );

		n = text.find_last_of(STRING(T("0")));				assert( n == 0 );
		n = text.find_last_of(STRING(T("a")));				assert( n == 10);
		n = text.find_last_of(STRING(T(".")));				assert( n == 39);
	  #ifdef _MSC_VER
		n = text.find_last_of(STRING(T("\0")));				assert( n == text.npos );
	  #endif
		n = text.find_last_of(STRING(T("?")));				assert( n == text.npos );
		n = text.find_last_of(STRING(T("")));				assert( n == text.npos );

		n = text.find_last_of(STRING(T("0")),10);			assert( n == 0 );
		n = text.find_last_of(STRING(T("a")),10);			assert( n == 10 );
		n = text.find_last_of(STRING(T(".")),10);			assert( n == text.npos );
	  #ifdef _MSC_VER
		n = text.find_last_of(STRING(T("\0")),10);			assert( n == text.npos );
	  #endif
		n = text.find_last_of(STRING(T("?")),10);			assert( n == text.npos );
		n = text.find_last_of(STRING(T("")),10);			assert( n == text.npos );
		n = text.find_first_of(STRING(T("0")),0);			assert( n == 0 );

		n = text.find_last_of(STRING(T("0123456789")));		assert( n == 9	);
		n = text.find_last_of(STRING(T("badcfe")));			assert( n == 15 );
		n = text.find_last_of(STRING(T(".#")));				assert( n == 39 );

		n = text.find_last_of(STRING(T("0123456789")),5);	assert( n == 5 );
		n = text.find_last_of(STRING(T("badcfe")), 5);		assert( n == text.npos);
		n = text.find_last_of(STRING(T("0123456789")), 9);	assert( n == 9 );
		n = text.find_last_of(STRING(T("badcfe")), 9);		assert( n == text.npos );
		n = text.find_last_of(STRING(T(".#")), 38 );		assert( n == 38 );
		n = text.find_first_of(STRING(T("a0")),0);			assert( n == 0 );

		text += STRING(text);

		n = text.find_last_of(T('0'));				assert( n == 40 );
		n = text.find_last_of(T('a'));				assert( n == 50 );
		n = text.find_last_of(T('.'));				assert( n == 79 );
	  #ifdef _MSC_VER
		n = text.find_last_of(T('\0'));				assert( n == text.npos );
	  #endif
		n = text.find_last_of(T('?'));				assert( n == text.npos );

		n = text.find_last_of(T('0'),10);			assert( n == 0 );
		n = text.find_last_of(T('a'),10);			assert( n == 10 );
		n = text.find_last_of(T('.'),10);			assert( n == text.npos );
	  #ifdef _MSC_VER
		n = text.find_last_of(T('\0'),10);			assert( n == text.npos );
	  #endif
		n = text.find_last_of(T('?'),10);			assert( n == text.npos );
		n = text.find_first_of(T('0'),0);			assert( n == 0 );

		n = text.find_last_of(T("0"));				assert( n == 40);
		n = text.find_last_of(T("a"));				assert( n == 50);
		n = text.find_last_of(T("."));				assert( n == 79);
		n = text.find_last_of(T("\0"));				assert( n == text.npos );
		n = text.find_last_of(T("?"));				assert( n == text.npos );

		n = text.find_last_of(T("0"),10);			assert( n == 0 );
		n = text.find_last_of(T("a"),10);			assert( n == 10 );
		n = text.find_last_of(T("."),10);			assert( n == text.npos );
		n = text.find_last_of(T("\0"),10);			assert( n == text.npos );
		n = text.find_last_of(T("?"),10);			assert( n == text.npos );
		n = text.find_first_of(T("0"),0);			assert( n == 0 );

		n = text.find_last_of(T("0123456789"));		assert( n == 49 );
		n = text.find_last_of(T("badcfe"));			assert( n == 55 );
		n = text.find_last_of(T(".#"));				assert( n == 79 );

		n = text.find_last_of(T("0123456789"),5);	assert( n == 5 );
		n = text.find_last_of(T("badcfe"), 5);		assert( n == text.npos);
		n = text.find_last_of(T("0123456789"), 9);	assert( n == 9 );
		n = text.find_last_of(T("badcfe"), 9);		assert( n == text.npos );
		n = text.find_last_of(T(".#"), 38 );		assert( n == 38 );
		n = text.find_first_of(T("a0"),0);			assert( n == 0 );

		n = text.find_last_of(STRING(T("0")));				assert( n == 40);
		n = text.find_last_of(STRING(T("a")));				assert( n == 50);
		n = text.find_last_of(STRING(T(".")));				assert( n == 79);
		n = text.find_last_of(STRING(T("\0")));				assert( n == text.npos );
		n = text.find_last_of(STRING(T("?")));				assert( n == text.npos );

		n = text.find_last_of(STRING(T("0")),10);			assert( n == 0 );
		n = text.find_last_of(STRING(T("a")),10);			assert( n == 10 );
		n = text.find_last_of(STRING(T(".")),10);			assert( n == text.npos );
		n = text.find_last_of(STRING(T("\0")),10);			assert( n == text.npos );
		n = text.find_last_of(STRING(T("?")),10);			assert( n == text.npos );
		n = text.find_first_of(STRING(T("0")),0);			assert( n == 0 );

		n = text.find_last_of(STRING(T("0123456789")));		assert( n == 49 );
		n = text.find_last_of(STRING(T("badcfe")));			assert( n == 55 );
		n = text.find_last_of(STRING(T(".#")));				assert( n == 79 );

		n = text.find_last_of(STRING(T("0123456789")),5);	assert( n == 5 );
		n = text.find_last_of(STRING(T("badcfe")), 5);		assert( n == text.npos);
		n = text.find_last_of(STRING(T("0123456789")), 9);	assert( n == 9 );
		n = text.find_last_of(STRING(T("badcfe")), 9);		assert( n == text.npos );
		n = text.find_last_of(STRING(T(".#")), 38 );		assert( n == 38 );
		n = text.find_first_of(STRING(T("a0")),0);			assert( n == 0 );
	}


	void test_find_first_not_of()
	{
		STRING	text =	T("0123456789")
						T("abcdefghij")
						T("klmnopqrst")
						T("uvwxyz###.");
		std::size_t n;
		n = text.find_first_not_of(T('0'));			assert( n == 1	);
		n = text.find_first_not_of(T('a'));			assert( n == 0	);
		n = text.find_first_not_of(T('.'));			assert( n == 0	);
		n = text.find_first_not_of(T('\0'));		assert( n == 0	);
		n = text.find_first_not_of(T('?'));			assert( n == 0	);

		n = text.find_first_not_of(T('0'),10);			assert( n == 10 );
		n = text.find_first_not_of(T('a'),10);			assert( n == 11 );
		n = text.find_first_not_of(T('.'),10);			assert( n == 10 );
		n = text.find_first_not_of(T('\0'),10);			assert( n == 10 );
		n = text.find_first_not_of(T('?'),10);			assert( n == 10 );
		n = text.find_first_not_of(T('0'),text.npos);	assert( n == text.npos );

		n = text.find_first_not_of(T("0"));				assert( n == 1	);
		n = text.find_first_not_of(T("a"));				assert( n == 0	);
		n = text.find_first_not_of(T("."));				assert( n == 0	);
		n = text.find_first_not_of(T("?"));				assert( n == 0	);
		//n = text.find_first_not_of(T("\0"));			assert( n == 0	);
		n = text.find_first_not_of(T(""));				assert( n == 0	);

		n = text.find_first_not_of(T("0"),10);			assert( n == 10 );
		n = text.find_first_not_of(T("a"),10);			assert( n == 11 );
		n = text.find_first_not_of(T("."),10);			assert( n == 10 );
		n = text.find_first_not_of(T("\0"),10);			assert( n == 10 );
		n = text.find_first_not_of(T("?"),10);			assert( n == 10 );
		n = text.find_first_not_of(T("0"),text.npos);	assert( n == text.npos );
		n = text.find_first_not_of(T(""), 10);			assert( n == 10 );

		n = text.find_first_not_of(T("0123456789"));	assert( n == 10 );
		n = text.find_first_not_of(T("badcfe"));		assert( n == 0	);
		n = text.find_first_not_of(T(".#"));			assert( n == 0	);

		n = text.find_first_not_of(T("0123456789"),5);	assert( n == 10 );
		n = text.find_first_not_of(T("badcfe"), 5);		assert( n == 5	);
		n = text.find_first_not_of(T("0123456789"),10);	assert( n == 10 );
		n = text.find_first_not_of(T("badcfe"), 10);	assert( n == 16 );
		n = text.find_first_not_of(T(".#"), 10);		assert( n == 10 );
		n = text.find_first_not_of(T("a0"),text.npos);	assert( n == text.npos );
		n = text.find_first_not_of(T(""),text.npos);	assert( n == text.npos );

		n = text.find_first_not_of(STRING(T("0")));			assert( n == 1	);
		n = text.find_first_not_of(STRING(T("a")));			assert( n == 0	);
		n = text.find_first_not_of(STRING(T(".")));			assert( n == 0	);
		n = text.find_first_not_of(STRING(T("?")));			assert( n == 0	);
		n = text.find_first_not_of(STRING(T("")));			assert( n == 0	);

		n = text.find_first_not_of(STRING(T("0")),10);			assert( n == 10 );
		n = text.find_first_not_of(STRING(T("a")),10);			assert( n == 11 );
		n = text.find_first_not_of(STRING(T(".")),10);			assert( n == 10 );
		n = text.find_first_not_of(STRING(T("\0")),10);			assert( n == 10 );
		n = text.find_first_not_of(STRING(T("?")),10);			assert( n == 10 );
		n = text.find_first_not_of(STRING(T("0")),text.npos);	assert( n == text.npos );
		n = text.find_first_not_of(STRING(T("")), 10);			assert( n == 10 );

		n = text.find_first_not_of(STRING(T("0123456789")));	assert( n == 10 );
		n = text.find_first_not_of(STRING(T("badcfe")));		assert( n == 0	);
		n = text.find_first_not_of(STRING(T(".#")));			assert( n == 0	);

		n = text.find_first_not_of(STRING(T("0123456789")),5);	assert( n == 10 );
		n = text.find_first_not_of(STRING(T("badcfe")), 5);		assert( n == 5	);
		n = text.find_first_not_of(STRING(T("0123456789")),10);	assert( n == 10 );
		n = text.find_first_not_of(STRING(T("badcfe")), 10);	assert( n == 16 );
		n = text.find_first_not_of(STRING(T(".#")), 10);		assert( n == 10 );
		n = text.find_first_not_of(STRING(T("a0")),text.npos);	assert( n == text.npos );
		n = text.find_first_not_of(STRING(T("")),text.npos);	assert( n == text.npos );

		text += STRING(text);

		n = text.find_first_not_of(T('0'));			assert( n == 1	);
		n = text.find_first_not_of(T('a'));			assert( n == 0	);
		n = text.find_first_not_of(T('.'));			assert( n == 0	);
		n = text.find_first_not_of(T('\0'));		assert( n == 0	);
		n = text.find_first_not_of(T('?'));			assert( n == 0	);

		n = text.find_first_not_of(T('0'),10);			assert( n == 10 );
		n = text.find_first_not_of(T('a'),10);			assert( n == 11 );
		n = text.find_first_not_of(T('.'),10);			assert( n == 10 );
		n = text.find_first_not_of(T('\0'),10);			assert( n == 10 );
		n = text.find_first_not_of(T('?'),10);			assert( n == 10 );
		n = text.find_first_not_of(T('0'),text.npos);	assert( n == text.npos );

		n = text.find_first_not_of(T("0"));				assert( n == 1	);
		n = text.find_first_not_of(T("a"));				assert( n == 0	);
		n = text.find_first_not_of(T("."));				assert( n == 0	);
		n = text.find_first_not_of(T("?"));				assert( n == 0	);
		//n = text.find_first_not_of(T("\0"));			assert( n == 0	);
		n = text.find_first_not_of(T(""));				assert( n == 0	);

		n = text.find_first_not_of(T("0"),10);			assert( n == 10 );
		n = text.find_first_not_of(T("a"),10);			assert( n == 11 );
		n = text.find_first_not_of(T("."),10);			assert( n == 10 );
		n = text.find_first_not_of(T("\0"),10);			assert( n == 10 );
		n = text.find_first_not_of(T("?"),10);			assert( n == 10 );
		n = text.find_first_not_of(T("0"),text.npos);	assert( n == text.npos );
		n = text.find_first_not_of(T(""), 10);			assert( n == 10 );

		n = text.find_first_not_of(T("0123456789"));	assert( n == 10 );
		n = text.find_first_not_of(T("badcfe"));		assert( n == 0	);
		n = text.find_first_not_of(T(".#"));			assert( n == 0	);

		n = text.find_first_not_of(T("0123456789"),5);	assert( n == 10 );
		n = text.find_first_not_of(T("badcfe"), 5);		assert( n == 5	);
		n = text.find_first_not_of(T("0123456789"),10);	assert( n == 10 );
		n = text.find_first_not_of(T("badcfe"), 10);	assert( n == 16 );
		n = text.find_first_not_of(T(".#"), 10);		assert( n == 10 );
		n = text.find_first_not_of(T("a0"),text.npos);	assert( n == text.npos );
		n = text.find_first_not_of(T(""),text.npos);	assert( n == text.npos );

		n = text.find_first_not_of(STRING(T("0")));			assert( n == 1	);
		n = text.find_first_not_of(STRING(T("a")));			assert( n == 0	);
		n = text.find_first_not_of(STRING(T(".")));			assert( n == 0	);
		n = text.find_first_not_of(STRING(T("?")));			assert( n == 0	);
		n = text.find_first_not_of(STRING(T("")));			assert( n == 0	);

		n = text.find_first_not_of(STRING(T("0")),10);			assert( n == 10 );
		n = text.find_first_not_of(STRING(T("a")),10);			assert( n == 11 );
		n = text.find_first_not_of(STRING(T(".")),10);			assert( n == 10 );
		n = text.find_first_not_of(STRING(T("\0")),10);			assert( n == 10 );
		n = text.find_first_not_of(STRING(T("?")),10);			assert( n == 10 );
		n = text.find_first_not_of(STRING(T("0")),text.npos);	assert( n == text.npos );
		n = text.find_first_not_of(STRING(T("")), 10);			assert( n == 10 );

		n = text.find_first_not_of(STRING(T("0123456789")));	assert( n == 10 );
		n = text.find_first_not_of(STRING(T("badcfe")));		assert( n == 0	);
		n = text.find_first_not_of(STRING(T(".#")));			assert( n == 0	);

		n = text.find_first_not_of(STRING(T("0123456789")),5);	assert( n == 10 );
		n = text.find_first_not_of(STRING(T("badcfe")), 5);		assert( n == 5	);
		n = text.find_first_not_of(STRING(T("0123456789")),10);	assert( n == 10 );
		n = text.find_first_not_of(STRING(T("badcfe")), 10);	assert( n == 16 );
		n = text.find_first_not_of(STRING(T(".#")), 10);		assert( n == 10 );
		n = text.find_first_not_of(STRING(T("a0")),text.npos);	assert( n == text.npos );
		n = text.find_first_not_of(STRING(T("")),text.npos);	assert( n == text.npos );
	}

	void test_find_last_not_of()
	{
		STRING	text =	T("0123456789")
						T("abcdefghij")
						T("klmnopqrst")
						T("uvwxyz###.");
		std::size_t n;

		n = text.find_last_not_of(T('0'));				assert( n == 39 );
		n = text.find_last_not_of(T('a'));				assert( n == 39 );
		n = text.find_last_not_of(T('.'));				assert( n == 38 );
		n = text.find_last_not_of(T('\0'));				assert( n == 39 );
		n = text.find_last_not_of(T('?'));				assert( n == 39 );

		n = text.find_last_not_of(T('0'),10);			assert( n == 10 );
		n = text.find_last_not_of(T('a'),10);			assert( n ==  9 );
		n = text.find_last_not_of(T('.'),10);			assert( n == 10 );
		n = text.find_last_not_of(T('\0'),10);			assert( n == 10 );
		n = text.find_last_not_of(T('?'),10);			assert( n == 10 );
		n = text.find_last_not_of(T('0'),text.npos);	assert( n == 39 );
		n = text.find_last_not_of(T('0'),0);			assert( n == text.npos );

		n = text.find_last_not_of(T("0"));				assert( n == 39 );
		n = text.find_last_not_of(T("a"));				assert( n == 39 );
		n = text.find_last_not_of(T("."));				assert( n == 38 );
		n = text.find_last_not_of(T("?"));				assert( n == 39 );
		n = text.find_last_not_of(T(""));				assert( n == 39 );

		n = text.find_last_not_of(T("0"),10);			assert( n == 10 );
		n = text.find_last_not_of(T("a"),10);			assert( n ==  9 );
		n = text.find_last_not_of(T("."),10);			assert( n == 10 );
		n = text.find_last_not_of(T("\0"),10);			assert( n == 10 );
		n = text.find_last_not_of(T("?"),10);			assert( n == 10 );
		n = text.find_last_not_of(T("0"),text.npos);	assert( n == 39 );

		n = text.find_last_not_of(T("0123456789"));				assert( n == 39 );
		n = text.find_last_not_of(T("badcfe"));					assert( n == 39 );
		n = text.find_last_not_of(T(".#"));						assert( n == 35 );

		n = text.find_last_not_of(T("0123456789"),5);			assert( n == text.npos );
		n = text.find_last_not_of(T("badcfe"), 5);				assert( n ==  5 );
		n = text.find_last_not_of(T("0123456789"),10);			assert( n == 10 );
		n = text.find_last_not_of(T("badcfe"), 10);				assert( n ==  9 );
		n = text.find_last_not_of(T(".#"), 10);					assert( n == 10 );
		n = text.find_last_not_of(T("a0"),text.npos);			assert( n == 39 );
		n = text.find_last_not_of(T(""),text.npos);				assert( n == 39 );
		n = text.find_last_not_of(T(""),0);						assert( n ==  0 );

		n = text.find_last_not_of(STRING(T("0")));				assert( n == 39 );
		n = text.find_last_not_of(STRING(T("a")));				assert( n == 39 );
		n = text.find_last_not_of(STRING(T(".")));				assert( n == 38 );
		n = text.find_last_not_of(STRING(T("?")));				assert( n == 39 );
		n = text.find_last_not_of(STRING(T("")));				assert( n == 39 );

		n = text.find_last_not_of(STRING(T("0")),10);			assert( n == 10 );
		n = text.find_last_not_of(STRING(T("a")),10);			assert( n ==  9 );
		n = text.find_last_not_of(STRING(T(".")),10);			assert( n == 10 );
		n = text.find_last_not_of(STRING(T("\0")),10);			assert( n == 10 );
		n = text.find_last_not_of(STRING(T("?")),10);			assert( n == 10 );
		n = text.find_last_not_of(STRING(T("0")),text.npos);	assert( n == 39 );

		n = text.find_last_not_of(STRING(T("0123456789")));		assert( n == 39 );
		n = text.find_last_not_of(STRING(T("badcfe")));			assert( n == 39 );
		n = text.find_last_not_of(STRING(T(".#")));				assert( n == 35 );

		n = text.find_last_not_of(STRING(T("0123456789")),5);	assert( n == text.npos );
		n = text.find_last_not_of(STRING(T("badcfe")), 5);		assert( n ==  5 );
		n = text.find_last_not_of(STRING(T("0123456789")),10);	assert( n == 10 );
		n = text.find_last_not_of(STRING(T("badcfe")), 10);		assert( n ==  9 );
		n = text.find_last_not_of(STRING(T(".#")), 10);			assert( n == 10 );
		n = text.find_last_not_of(STRING(T("a0")),text.npos);	assert( n == 39 );
		n = text.find_last_not_of(STRING(T("")),text.npos);		assert( n == 39 );
		n = text.find_last_not_of(STRING(T("")),0);				assert( n ==  0 );

		text += STRING(text);

		n = text.find_last_not_of(T('0'));				assert( n == 79 );
		n = text.find_last_not_of(T('a'));				assert( n == 79 );
		n = text.find_last_not_of(T('.'));				assert( n == 78 );
		n = text.find_last_not_of(T('\0'));				assert( n == 79 );
		n = text.find_last_not_of(T('?'));				assert( n == 79 );

		n = text.find_last_not_of(T('0'),10);			assert( n == 10 );
		n = text.find_last_not_of(T('a'),10);			assert( n ==  9 );
		n = text.find_last_not_of(T('.'),10);			assert( n == 10 );
		n = text.find_last_not_of(T('\0'),10);			assert( n == 10 );
		n = text.find_last_not_of(T('?'),10);			assert( n == 10 );
		n = text.find_last_not_of(T('0'),text.npos);	assert( n == 79 );
		n = text.find_last_not_of(T('0'),0);			assert( n == text.npos );

		n = text.find_last_not_of(T("0"));				assert( n == 79 );
		n = text.find_last_not_of(T("a"));				assert( n == 79 );
		n = text.find_last_not_of(T("."));				assert( n == 78 );
		n = text.find_last_not_of(T("?"));				assert( n == 79 );
		n = text.find_last_not_of(T(""));				assert( n == 79 );

		n = text.find_last_not_of(T("0"),10);			assert( n == 10 );
		n = text.find_last_not_of(T("a"),10);			assert( n ==  9 );
		n = text.find_last_not_of(T("."),10);			assert( n == 10 );
		n = text.find_last_not_of(T("\0"),10);			assert( n == 10 );
		n = text.find_last_not_of(T("?"),10);			assert( n == 10 );
		n = text.find_last_not_of(T("0"),text.npos);	assert( n == 79 );

		n = text.find_last_not_of(T("0123456789"));		assert( n == 79 );
		n = text.find_last_not_of(T("badcfe"));			assert( n == 79 );
		n = text.find_last_not_of(T(".#"));				assert( n == 75 );

		n = text.find_last_not_of(T("0123456789"),5);	assert( n == text.npos );
		n = text.find_last_not_of(T("badcfe"), 5);		assert( n ==  5 );
		n = text.find_last_not_of(T("0123456789"),10);	assert( n == 10 );
		n = text.find_last_not_of(T("badcfe"), 10);		assert( n ==  9 );
		n = text.find_last_not_of(T(".#"), 10);			assert( n == 10 );
		n = text.find_last_not_of(T("a0"),text.npos);	assert( n == 79 );
		n = text.find_last_not_of(T(""),text.npos);		assert( n == 79 );
		n = text.find_last_not_of(T(""),0);				assert( n ==  0 );

		n = text.find_last_not_of(STRING(T("0")));				assert( n == 79 );
		n = text.find_last_not_of(STRING(T("a")));				assert( n == 79 );
		n = text.find_last_not_of(STRING(T(".")));				assert( n == 78 );
		n = text.find_last_not_of(STRING(T("?")));				assert( n == 79 );
		n = text.find_last_not_of(STRING(T("")));				assert( n == 79 );

		n = text.find_last_not_of(STRING(T("0")),10);			assert( n == 10 );
		n = text.find_last_not_of(STRING(T("a")),10);			assert( n ==  9 );
		n = text.find_last_not_of(STRING(T(".")),10);			assert( n == 10 );
		n = text.find_last_not_of(STRING(T("\0")),10);			assert( n == 10 );
		n = text.find_last_not_of(STRING(T("?")),10);			assert( n == 10 );
		n = text.find_last_not_of(STRING(T("0")),text.npos);	assert( n == 79 );

		n = text.find_last_not_of(STRING(T("0123456789")));		assert( n == 79 );
		n = text.find_last_not_of(STRING(T("badcfe")));			assert( n == 79 );
		n = text.find_last_not_of(STRING(T(".#")));				assert( n == 75 );

		n = text.find_last_not_of(STRING(T("0123456789")),5);	assert( n == text.npos );
		n = text.find_last_not_of(STRING(T("badcfe")), 5);		assert( n ==  5 );
		n = text.find_last_not_of(STRING(T("0123456789")),10);	assert( n == 10 );
		n = text.find_last_not_of(STRING(T("badcfe")), 10);		assert( n ==  9 );
		n = text.find_last_not_of(STRING(T(".#")), 10);			assert( n == 10 );
		n = text.find_last_not_of(STRING(T("a0")),text.npos);	assert( n == 79 );
		n = text.find_last_not_of(STRING(T("")),text.npos);		assert( n == 79 );
		n = text.find_last_not_of(STRING(T("")),0);				assert( n ==  0 );
	}
};

#undef	TEST_OUT_OF_RANGE


#endif
